'use strict';

Object.defineProperty(exports, '__esModule', { value: true });

var helper = require('@tarojs/helper');
var runnerUtils = require('@tarojs/runner-utils');
var shared = require('@tarojs/shared');
var template = require('@tarojs/shared/dist/template');
var lodash = require('lodash');

function getLoaderMeta() {
    return {
        importFrameworkStatement: `
import Vue from 'vue';
`,
        mockAppStatement: `
const App = {
  render (h) {
    // this.$slots.default 是将要会渲染的页面
    return h('block', this.$slots.default)
  }
}
`,
        frameworkArgs: 'Vue, config',
        creator: 'createVueApp',
        creatorLocation: '@tarojs/plugin-framework-vue2/dist/runtime',
        importFrameworkName: 'Vue',
        isNeedRawLoader: true,
        extraImportForWeb: '',
        execBeforeCreateWebApp: '',
    };
}

const CUSTOM_WRAPPER = 'custom-wrapper';
function getVueLoaderPath$1() {
    try {
        return require.resolve('vue-loader', {
            paths: [process.cwd()]
        });
    }
    catch (error) {
        // eslint-disable-next-line no-console
        console.log(helper.chalk.yellow('找不到 vue-loader，请先安装。'));
        process.exit(1);
    }
}
function getH5VueLoaderOptions() {
    const h5Options = {
        transformAssetUrls: {
            video: ['src', 'poster'],
            'live-player': 'src',
            audio: 'src',
            source: 'src',
            image: 'src',
            'cover-image': 'src',
            'taro-video': ['src', 'poster'],
            'taro-live-player': 'src',
            'taro-audio': 'src',
            'taro-source': 'src',
            'taro-image': 'src',
            'taro-cover-image': 'src'
        },
        compilerOptions: {
            modules: [{
                    preTransformNode(el) {
                        if (runnerUtils.DEFAULT_Components.has(el.tag)) {
                            el.tag = 'taro-' + el.tag;
                        }
                        return el;
                    }
                }]
        }
    };
    return h5Options;
}
function getMiniVueLoaderOptions(componentConfig) {
    const miniOptions = {
        optimizeSSR: false,
        transformAssetUrls: {
            video: ['src', 'poster'],
            'live-player': 'src',
            audio: 'src',
            source: 'src',
            image: 'src',
            'cover-image': 'src'
        },
        compilerOptions: {
            whitespace: 'condense',
            modules: [{
                    preTransformNode(el) {
                        const nodeName = el.tag;
                        if (shared.capitalize(shared.toCamelCase(nodeName)) in shared.internalComponents) {
                            componentConfig.includes.add(nodeName);
                        }
                        if (nodeName === CUSTOM_WRAPPER) {
                            componentConfig.thirdPartyComponents.set(CUSTOM_WRAPPER, new Set());
                        }
                        const usingComponent = componentConfig.thirdPartyComponents.get(nodeName);
                        if (usingComponent != null) {
                            el.attrsList
                                .filter(a => !a.dynamic)
                                .forEach(a => usingComponent.add(a.name.startsWith(':') ? a.name.slice(1) : a.name));
                        }
                        return el;
                    }
                }],
            mustUseProp: function () {
                return false;
            }
        }
    };
    return miniOptions;
}
function getHarmonyVueLoaderOptions(componentConfig) {
    const harmonyOptions = {
        optimizeSSR: false,
        transformAssetUrls: {
            video: ['src', 'poster'],
            'live-player': 'src',
            audio: 'src',
            source: 'src',
            image: 'src',
            'cover-image': 'src'
        },
        compilerOptions: {
            whitespace: 'condense',
            modules: [{
                    preTransformNode(el) {
                        const nodeName = el.tag;
                        if (shared.capitalize(shared.toCamelCase(nodeName)) in shared.internalComponents) {
                            componentConfig.includes.add(nodeName);
                        }
                        if (nodeName === CUSTOM_WRAPPER) {
                            componentConfig.thirdPartyComponents.set(CUSTOM_WRAPPER, new Set());
                        }
                        const usingComponent = componentConfig.thirdPartyComponents.get(nodeName);
                        if (usingComponent != null) {
                            el.attrsList
                                .filter(a => !a.dynamic)
                                .forEach(a => usingComponent.add(a.name.startsWith(':') ? a.name.slice(1) : a.name));
                        }
                        return el;
                    }
                }],
            mustUseProp: function () {
                return false;
            }
        }
    };
    return harmonyOptions;
}

/******************************************************************************
Copyright (c) Microsoft Corporation.

Permission to use, copy, modify, and/or distribute this software for any
purpose with or without fee is hereby granted.

THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL WARRANTIES WITH
REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY
AND FITNESS. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY SPECIAL, DIRECT,
INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING FROM
LOSS OF USE, DATA OR PROFITS, WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE OR
OTHER TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION WITH THE USE OR
PERFORMANCE OF THIS SOFTWARE.
***************************************************************************** */
/* global Reflect, Promise, SuppressedError, Symbol */


function __awaiter(thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
}

typeof SuppressedError === "function" ? SuppressedError : function (error, suppressed, message) {
    var e = new Error(message);
    return e.name = "SuppressedError", e.error = error, e.suppressed = suppressed, e;
};

function h5VitePlugin(ctx) {
    // eslint-disable-next-line no-console
    return [
        // @TODO: 确认 webpack.h5 的 customStyle 逻辑是否需要迁移
        injectLoaderMeta$2(ctx),
        require('@vitejs/plugin-vue2').default({
            template: getH5VueLoaderOptions()
        })
    ];
}
function injectLoaderMeta$2(ctx) {
    function customizer(object = '', sources = '') {
        if ([object, sources].every(e => typeof e === 'string'))
            return object + sources;
    }
    const { runnerUtils } = ctx;
    const { getViteH5CompilerContext } = runnerUtils;
    return {
        name: 'taro-vue3:loader-meta',
        buildStart() {
            return __awaiter(this, void 0, void 0, function* () {
                const viteCompilerContext = yield getViteH5CompilerContext(this);
                if (viteCompilerContext) {
                    viteCompilerContext.loaderMeta = lodash.mergeWith(getLoaderMeta(), viteCompilerContext.loaderMeta, customizer);
                }
            });
        }
    };
}

function modifyHarmonyWebpackChain(chain, data) {
    setVueLoader$2(chain, data);
    setLoader$3(chain);
}
function setVueLoader$2(chain, data) {
    const vueLoaderPath = getVueLoaderPath$1();
    // plugin
    const { VueLoaderPlugin } = require(vueLoaderPath);
    chain
        .plugin('vueLoaderPlugin')
        .use(VueLoaderPlugin);
    // loader
    const vueLoaderOption = getHarmonyVueLoaderOptions(data.componentConfig);
    chain.module
        .rule('vue')
        .test(helper.REG_VUE)
        .use('vueLoader')
        .loader(vueLoaderPath)
        .options(vueLoaderOption);
}
function setLoader$3(chain) {
    chain.plugin('mainPlugin')
        .tap(args => {
        args[0].loaderMeta = getLoaderMeta();
        return args;
    });
}

function harmonyVitePlugin(ctx, componentConfig) {
    return [
        injectLoaderMeta$1(ctx),
        require('@vitejs/plugin-vue2').default({
            template: getHarmonyVueLoaderOptions(componentConfig)
        })
    ];
}
function injectLoaderMeta$1(ctx) {
    return {
        name: 'taro-vue3:loader-meta',
        buildStart() {
            const { runnerUtils } = ctx;
            const { getViteHarmonyCompilerContext } = runnerUtils;
            const viteCompilerContext = getViteHarmonyCompilerContext(this);
            if (viteCompilerContext) {
                viteCompilerContext.loaderMeta = getLoaderMeta();
            }
        },
    };
}

function modifyMiniWebpackChain(chain, data) {
    setVueLoader$1(chain, data);
    setLoader$2(chain);
}
function setVueLoader$1(chain, data) {
    const vueLoaderPath = getVueLoaderPath$1();
    // plugin
    const { VueLoaderPlugin } = require(vueLoaderPath);
    chain
        .plugin('vueLoaderPlugin')
        .use(VueLoaderPlugin);
    // loader
    const vueLoaderOption = getMiniVueLoaderOptions(data.componentConfig);
    chain.module
        .rule('vue')
        .test(helper.REG_VUE)
        .use('vueLoader')
        .loader(vueLoaderPath)
        .options(vueLoaderOption);
}
function setLoader$2(chain) {
    chain.plugin('miniPlugin')
        .tap(args => {
        args[0].loaderMeta = getLoaderMeta();
        return args;
    });
}

function miniVitePlugin(ctx, componentConfig) {
    return [
        injectLoaderMeta(ctx),
        require('@vitejs/plugin-vue2').default({
            template: getMiniVueLoaderOptions(componentConfig)
        })
    ];
}
function injectLoaderMeta(ctx) {
    return {
        name: 'taro-vue3:loader-meta',
        buildStart() {
            const { runnerUtils } = ctx;
            const { getViteMiniCompilerContext } = runnerUtils;
            const viteCompilerContext = getViteMiniCompilerContext(this);
            if (viteCompilerContext) {
                viteCompilerContext.loaderMeta = getLoaderMeta();
            }
        },
    };
}

function modifyH5WebpackChain(ctx, chain) {
    var _a;
    setStyleLoader(ctx, chain);
    setVueLoader(chain);
    setLoader$1(chain);
    const { isBuildNativeComp = false } = ((_a = ctx.runOpts) === null || _a === void 0 ? void 0 : _a.options) || {};
    const externals = {};
    if (isBuildNativeComp) {
        // Note: 该模式不支持 prebundle 优化，不必再处理
        externals.vue = 'vue';
    }
    chain.merge({ externals });
}
function setStyleLoader(ctx, chain) {
    const config = ctx.initialConfig.h5 || {};
    const { styleLoaderOption = {} } = config;
    chain.module
        .rule('customStyle')
        .merge({
        use: [{
                loader: 'style-loader',
                options: styleLoaderOption
            }]
    });
}
function setVueLoader(chain) {
    const vueLoaderPath = getVueLoaderPath$1();
    // plugin
    const { VueLoaderPlugin } = require(vueLoaderPath);
    chain
        .plugin('vueLoaderPlugin')
        .use(VueLoaderPlugin);
    // loader
    const vueLoaderOption = getH5VueLoaderOptions();
    chain.module
        .rule('vue')
        .test(helper.REG_VUE)
        .use('vueLoader')
        .loader(vueLoaderPath)
        .options(vueLoaderOption);
}
function setLoader$1(chain) {
    function customizer(object = '', sources = '') {
        if ([object, sources].every((e) => typeof e === 'string'))
            return object + sources;
    }
    chain.plugin('mainPlugin')
        .tap(args => {
        args[0].loaderMeta = lodash.mergeWith(getLoaderMeta(), args[0].loaderMeta, customizer);
        return args;
    });
}

var index = (ctx) => {
    const { framework } = ctx.initialConfig;
    if (framework !== 'vue')
        return;
    ctx.modifyWebpackChain(({ chain, data }) => {
        var _a;
        // 通用
        if (process.env.NODE_ENV !== 'production') {
            setAlias(chain);
        }
        customVueChain(chain, data);
        setLoader(chain);
        if (process.env.TARO_PLATFORM === 'web') {
            // H5
            modifyH5WebpackChain(ctx, chain);
        }
        else if (process.env.TARO_PLATFORM === 'harmony' || process.env.TARO_ENV === 'harmony') {
            // 鸿蒙
            modifyHarmonyWebpackChain(ctx, data);
        }
        else {
            // 小程序
            modifyMiniWebpackChain(chain, data);
        }
        const { isBuildNativeComp = false } = ((_a = ctx.runOpts) === null || _a === void 0 ? void 0 : _a.options) || {};
        const externals = {};
        if (isBuildNativeComp) {
            // Note: 该模式不支持 prebundle 优化，不必再处理
            externals.vue = 'vue';
        }
        chain.merge({ externals });
    });
    ctx.modifyViteConfig(({ viteConfig, data }) => {
        viteConfig.plugins.push(viteCommonPlugin());
        if (process.env.TARO_PLATFORM === 'web') {
            // H5
            viteConfig.plugins.push(h5VitePlugin(ctx));
        }
        else if (process.env.TARO_PLATFORM === 'harmony' || process.env.TARO_ENV === 'harmony') {
            // 鸿蒙
            viteConfig.plugins.push(harmonyVitePlugin(ctx, data === null || data === void 0 ? void 0 : data.componentConfig));
        }
        else {
            // 小程序
            viteConfig.plugins.push(miniVitePlugin(ctx, data === null || data === void 0 ? void 0 : data.componentConfig));
        }
    });
    ctx.modifyRunnerOpts(({ opts }) => {
        opts.frameworkExts = helper.VUE_EXT;
        if (!(opts === null || opts === void 0 ? void 0 : opts.compiler))
            return;
        if (shared.isString(opts.compiler)) {
            opts.compiler = {
                type: opts.compiler
            };
        }
        const { compiler } = opts;
        if (compiler.type === 'webpack5') {
            // 提供给 webpack5 依赖预编译收集器的第三方依赖
            const deps = [
                'vue',
                '@tarojs/plugin-framework-vue2/dist/runtime'
            ];
            compiler.prebundle || (compiler.prebundle = {});
            const prebundleOptions = compiler.prebundle;
            prebundleOptions.include || (prebundleOptions.include = []);
            prebundleOptions.include = prebundleOptions.include.concat(deps);
            prebundleOptions.exclude || (prebundleOptions.exclude = []);
        }
    });
    ctx.onParseCreateElement(({ nodeName, componentConfig }) => {
        if (template.capitalize(template.toCamelCase(nodeName)) in template.internalComponents) {
            componentConfig.includes.add(nodeName);
        }
    });
};
function getVueLoaderPath() {
    try {
        return require.resolve('vue-loader', {
            paths: [process.cwd()]
        });
    }
    catch (error) {
        // eslint-disable-next-line no-console
        console.log(helper.chalk.yellow('找不到 vue-loader，请先安装。'));
        process.exit(1);
    }
}
function customVueChain(chain, data) {
    const vueLoaderPath = getVueLoaderPath();
    // plugin
    const { VueLoaderPlugin } = require(vueLoaderPath);
    chain
        .plugin('vueLoaderPlugin')
        .use(VueLoaderPlugin);
    // loader
    let vueLoaderOption;
    if (process.env.TARO_PLATFORM === 'web') {
        // H5
        vueLoaderOption = {
            transformAssetUrls: {
                video: ['src', 'poster'],
                'live-player': 'src',
                audio: 'src',
                source: 'src',
                image: 'src',
                'cover-image': 'src',
                'taro-video': ['src', 'poster'],
                'taro-live-player': 'src',
                'taro-audio': 'src',
                'taro-source': 'src',
                'taro-image': 'src',
                'taro-cover-image': 'src'
            },
            compilerOptions: {
                modules: [{
                        preTransformNode(el) {
                            if (runnerUtils.DEFAULT_Components.has(el.tag)) {
                                el.tag = 'taro-' + el.tag;
                            }
                            return el;
                        }
                    }]
            }
        };
    }
    else {
        // 小程序
        vueLoaderOption = {
            optimizeSSR: false,
            transformAssetUrls: {
                video: ['src', 'poster'],
                'live-player': 'src',
                audio: 'src',
                source: 'src',
                image: 'src',
                'cover-image': 'src'
            },
            compilerOptions: {
                whitespace: 'condense',
                modules: [{
                        preTransformNode(el) {
                            const nodeName = el.tag;
                            if (template.capitalize(template.toCamelCase(nodeName)) in template.internalComponents) {
                                data.componentConfig.includes.add(nodeName);
                            }
                            if (nodeName === CUSTOM_WRAPPER) {
                                data.componentConfig.thirdPartyComponents.set(CUSTOM_WRAPPER, new Set());
                            }
                            const usingComponent = data.componentConfig.thirdPartyComponents.get(nodeName);
                            if (usingComponent != null) {
                                el.attrsList
                                    .filter(a => !a.dynamic)
                                    .forEach(a => usingComponent.add(a.name.startsWith(':') ? a.name.slice(1) : a.name));
                            }
                            return el;
                        }
                    }],
                mustUseProp: function () {
                    return false;
                }
            }
        };
    }
    chain.module
        .rule('vue')
        .test(helper.REG_VUE)
        .use('vueLoader')
        .loader(vueLoaderPath)
        .options(vueLoaderOption);
}
function setLoader(chain) {
    function customizer(object = '', sources = '') {
        if ([object, sources].every(e => typeof e === 'string'))
            return object + sources;
    }
    if (process.env.TARO_PLATFORM === 'web') {
        chain.plugin('mainPlugin')
            .tap(args => {
            args[0].loaderMeta = lodash.mergeWith(getLoaderMeta(), args[0].loaderMeta, customizer);
            return args;
        });
    }
    else {
        chain.plugin('miniPlugin')
            .tap(args => {
            args[0].loaderMeta = getLoaderMeta();
            return args;
        });
    }
}
function setAlias(chain) {
    // 避免 npm link 时，taro composition apis 使用的 vue 和项目使用的 vue 实例不一致。
    chain.resolve.alias
        .set('vue', require.resolve('vue'));
}
function viteCommonPlugin() {
    return {
        name: 'taro-vue2:common',
        config: () => ({
            resolve: {
                dedupe: process.env.NODE_ENV !== 'production' ? ['vue'] : []
            }
        })
    };
}

exports.default = index;
//# sourceMappingURL=index.js.map
